@@ -1,10 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 import argparse
 import logging
 import os
 from pathlib import Path
-import shutil
 import subprocess
 import sys
 from createpythonpackage import templates
@@ -13,6 +12,8 @@ TEST_PYPI_URL = "https://test.pypi.org/simple/"
 ISATTY = sys.stdout.isatty()
 WINDOWS = os.name == "nt"
 
+__version__ = "0.1.0.0"
+
 
 def blue(text):
     if ISATTY and not WINDOWS:
@@ -25,7 +26,7 @@ def printblue(text):
 
 
 def print_version():
-    print("0.0.0.5")
+    print(__version__)
 
 
 class CppError(Exception):
@@ -41,13 +42,31 @@ def _run(cmd, check=True, **kwargs):
     return returncode
 
 
+def _create_venv(path: Path, name: str, python: str = sys.executable):
+    venv = path / "venv"
+    if venv.exists() and len(list(venv.iterdir())):
+        raise CppError(
+            f"Directory {str(venv)!r} already exists. Remove then try again."
+        )
+    print(f"Creating a virtual environment at {blue(str(venv))}")
+    if _run([python, "-m", "venv", venv, "--prompt", name]):
+        raise CppError("Could not create virtual environment")
+    print(f"Upgrading {blue('pip')} in the virtual environment.")
+    if _run([path / "venv/bin/pip", "install", "--upgrade", "--quiet", "pip"]):
+        logging.warning("Could not upgrade pip to latest verions")
+    print()
+    return venv
+
+
 class Package:
-    def __init__(self, path, name):
+    def __init__(self, path, name, author, version, email):
         self.path = path
         self.name = name
+        self.author = author
+        self.version = version
+        self.email = email
         self.add_files()
-        self.create_venv()
-        self.symlink_venv()
+        _create_venv(self.path, self.name)
         self.git_init()
 
     def git_init(self):
@@ -55,30 +74,22 @@ class Package:
         print("Initialized a git repository.")
         print()
 
-    def create_venv(self):
-        venv = self.path / "venv"
-        print(f"Creating a virtual environment at {blue(str(venv))}")
-        _run(["python3", "-m", "venv", venv, "--prompt", self.name])
-        _run([self.path / "venv/bin/pip", "install", "--upgrade", "--quiet", "pip"])
-        print(f"Upgrading {blue('pip')} in the virtual environment.")
-        print()
-
-    def symlink_venv(self):
-        relative_path = (self.path / "venv/bin/activate").relative_to(self.path)
-        (self.path / "activate-venv").symlink_to(relative_path)
-
     def add_files(self):
         (self.path / self.name).mkdir(exist_ok=True)
         (self.path / self.name / "__init__.py").touch()
 
         with open(self.path / "README.md", "w") as f:
-            f.write(templates.readme % self.name)
+            f.write(templates.readme.format(name=self.name, author=self.author))
 
         with open(self.path / "setup.py", "w") as f:
-            f.write(templates.setup % self.name)
+            f.write(
+                templates.setup.format(
+                    name=self.name, author=self.author, email=self.email
+                )
+            )
 
         with open(self.path / self.name / "main.py", "w") as f:
-            f.write(templates.main)
+            f.write(templates.main.format(version=self.version))
 
         with open(self.path / ".gitignore", "w") as f:
             f.write(templates.gitignore)
@@ -86,11 +97,14 @@ class Package:
         with open(self.path / "LICENSE", "w") as f:
             f.write(templates.licence)
 
+        with open(self.path / "makefile", "w") as f:
+            f.write(templates.makefile)
+
     def print_success(self):
         print(f"Success! Created {self.name} at {str(self.path)}")
         print("Inside that directory, you can run several commands")
         print()
-        printblue("  source activate-venv")
+        printblue("  source ven/bin/activate")
         print("     Activates this package's isolated Python environment")
         print()
         printblue("  pip install PACKAGE")
@@ -102,7 +116,7 @@ class Package:
         print(
             f"We suggest that you being by typing:\n\n"
             f"  {blue('cd')} {self.name}\n"
-            f"  {blue('source activate-venv')}\n\n"
+            f"  {blue('source ven/bin/activate')}\n\n"
             "To deactivate the virtual environment, type `deactivate`.\n"
         )
         print(
@@ -119,7 +133,9 @@ def mkdir(path):
     path.mkdir(parents=True, exist_ok=True)
 
 
-def _create(args):
+def _create_package(args):
+    author = args.author
+    version = args.packageversion
     name = Path(args.name).name
     path = Path(args.name).resolve()
     if Path(args.name).exists() and len(list(path.iterdir())):
@@ -127,7 +143,7 @@ def _create(args):
     print(f"Creating a new Python package in {blue(str(path))}")
     print()
     mkdir(path)
-    return Package(path, name)
+    return Package(path, name, author, version, args.email)
 
 
 def _setup_logging(verbose):
@@ -140,11 +156,58 @@ def _setup_logging(verbose):
         logging.basicConfig(level=logging.WARNING, format="%(message)s")
 
 
-def create():
-    parser = argparse.ArgumentParser()
+def _build_package(path):
+    if (path / "dist").exists() and list((path / "dist").iterdir()):
+        raise CppError(
+            f"Remove {str(path/'dist')!r} directory before building "
+            "to ensure a clean build"
+        )
+    print(
+        "running instructions from https://packaging.python.org/tutorials/packaging-projects/"
+    )
+    print(f"Upgrading {blue('setuptools')}, {blue('wheel')}, and {blue('twine')}")
+    cmd = ["pip", "install", "--upgrade", "--quiet", "setuptools", "wheel", "twine"]
+    print(f"  {' '.join(list(str(c) for c in cmd))}")
+    print()
+    _run(cmd)
+
+    cmd = ["python", path / "setup.py", "--quiet", "sdist", "bdist_wheel"]
+    printblue("Building package")
+    print(f"  {' '.join(list(str(c) for c in cmd))}")
+    print()
+    _run(cmd)
+    printblue("Package has been built! See output in 'dist' directory.")
+
+
+def _publish(path, test):
+    cmd = ["python", "-m", "twine", "upload"]
+    if test:
+        cmd += ["--repository-url", TEST_PYPI_URL]
+    cmd += ["dist/*"]
+    printblue("Uploading package")
+    print(f"  {' '.join(list(str(c) for c in cmd))}")
+    print()
+    _run(cmd)
+
+
+def create_package():
+    parser = argparse.ArgumentParser(
+        description=(
+            "Creates boilerplate for Python projects. Includes setup.py, "
+            "git repository, .gitinore, license, README, makefile."
+        ),
+        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+    )
     parser.add_argument("name", nargs="?", help="Name of package to create")
-    parser.add_argument("--verbose", action="store_true")
+    parser.add_argument("--author", help="Name of author", default="Your Name")
+    parser.add_argument(
+        "--email", help="Email of author", default="youremail@domain.com"
+    )
+    parser.add_argument(
+        "--packageversion", help="Initial version of your package", default="0.0.0.1"
+    )
     parser.add_argument("--version", action="store_true")
+    parser.add_argument("--verbose", action="store_true")
     args = parser.parse_args()
 
     if args.version:
@@ -156,44 +219,84 @@ def create():
     _setup_logging(args.verbose)
 
     try:
-        package = _create(args)
+        package = _create_package(args)
         package.print_success()
         exit(0)
     except CppError as e:
         exit(e)
 
 
-def _publish(path, test):
-    if (path / "dist").exists() and list((path / "dist").iterdir()):
-        raise CppError(f"Remove {str(path/'dist')} before publishing")
-    print(
-        "running instructions from https://packaging.python.org/tutorials/packaging-projects/"
+def build_package():
+    parser = argparse.ArgumentParser(
+        description="build a package but do not publish it"
     )
-    print(f"Upgrading {blue('setuptools')}, {blue('wheel')}, and {blue('twine')}")
-    cmd = ["pip", "install", "--upgrade", "--quiet", "setuptools", "wheel", "twine"]
-    print(f"  {' '.join(list(str(c) for c in cmd))}")
-    print()
-    _run(cmd)
+    parser.add_argument(
+        "path", help="Path to root of package (setup.py should be in this dir)"
+    )
+    parser.add_argument("--verbose", action="store_true")
+
+    args = parser.parse_args()
+    _setup_logging(args.verbose)
+    try:
+        path = Path(args.path).resolve()
+        if not path.is_dir():
+            exit(f"Directory {args.path} does not exist")
+        elif not (path / "setup.py").is_file():
+            exit(f"{str(path / 'setup.py')} does not exist")
+        _build_package(Path(args.path))
+    except CppError as e:
+        exit(e)
 
 
-    cmd = ["python", path / "setup.py", "--quiet", "sdist", "bdist_wheel"]
-    printblue("Building package")
-    print(f"  {' '.join(list(str(c) for c in cmd))}")
-    print()
-    _run(cmd)
+def create_venv():
+    parser = argparse.ArgumentParser(
+        description="create a virtual environment for the given directory"
+    )
+    parser.add_argument(
+        "path",
+        default=".",
+        help="Path where venv should be created (usually the root of a package next to setup.py)",
+    )
+    parser.add_argument(
+        "--python",
+        default=sys.executable,
+        help="Python binary this venv should be associated with",
+    )
+    parser.add_argument("--verbose", action="store_true")
 
-    cmd = ["python", "-m", "twine", "upload"]
-    if test:
-        cmd += ["--repository-url", TEST_PYPI_URL]
-    cmd += ["dist/*"]
-    printblue("Uploading package")
-    print(f"  {' '.join(list(str(c) for c in cmd))}")
-    print()
-    _run(cmd)
+    args = parser.parse_args()
+    _setup_logging(args.verbose)
+    try:
+        path = Path(args.path).resolve()
+        mkdir(path)
+        venv = _create_venv(path, path.name, args.python)
+        symlink_venv(path)
+        if (path / "requirements.txt").exists():
+            logging.info("Installing from requirements.txt file")
+            _run(
+                [
+                    venv / "bin" / "python",
+                    "-m",
+                    "pip",
+                    "install",
+                    "-r",
+                    path / "requirements.txt",
+                ]
+            )
+        printblue("A new virtual environment has been created!")
+        print()
+        print(
+            f"Run cd {str(path)!r} then type `source venv/bin/activate` "
+            "to activate it. When you are finished type `deactivate` to exit the environment."
+        )
+    except CppError as e:
+        exit(e)
 
 
 def publish():
-    parser = argparse.ArgumentParser()
+    parser = argparse.ArgumentParser(
+        description="builds an publishes a package to PyPI"
+    )
     parser.add_argument(
         "path",
         nargs="?",
@@ -223,11 +326,11 @@ def publish():
             exit(f"Directory {args.path} does not exist")
         elif not (path / "setup.py").is_file():
             exit(f"{str(path / 'setup.py')} does not exist")
+        _build_package(path)
         _publish(path, args.test)
-        exit(0)
     except CppError as e:
         exit(e)
 
 
 if __name__ == "__main__":
-    create()
+    create_package()
